      PROGRAM DRVCUB
C     ********************************************
C     * Driver: compute the hyperinterpolation 
C     * on the cube [0,1]^3 for the Renka test set
C     ********************************************
      implicit none
      INTEGER DEGMAX,NXUMAX,NTGMAX,number
      PARAMETER (DEGMAX = 40,NXUMAX = (DEGMAX + 2) * (DEGMAX + 2) / 2,
     1     NTGMAX = 8000)
      INTEGER DEG,NXU,NTG,count
      DOUBLE PRECISION XU1(NXUMAX),XU2(NXUMAX),WXU(NXUMAX),
     1     FVAL(NXUMAX,0:degmax+1),W(NXUMAX,0:degmax + 1)
      DOUBLE PRECISION T1(0:DEGMAX,NXUMAX),T2(0:DEGMAX,NXUMAX),
     1     t3(0:degmax)
      DOUBLE PRECISION C0(0:DEGMAX,0:DEGMAX,0:DEGMAX),
     1     B0(0:DEGMAX,0:DEGMAX)
      data c0/(degmax+1)**3 * 0.0d0/
      DOUBLE PRECISION TG1(NTGMAX),TG2(NTGMAX),TG3(NTGMAX),
     1     INTFTG(NTGMAX)
      DOUBLE PRECISION F,MAXERR,MAP1,MAP2,MAP3,XU2VAL,esterr,hy3val
      EXTERNAL F,MAP1,MAP2,XU2VAL,MAP3,hy3val
      INTEGER I,J,NTG1,k
      DOUBLE PRECISION pi
      PARAMETER (pi= 3.1415926535897931D0)
      real time
C     **********************************************************************
C     * LOCALS type                
C     *
C     * DEGMAX integer                   maximum degree allowed
C     * NXUMAX integer                   maximum number of Xu points
C     *                                  =(DEGMAX+2)*(DEGMAX+2)/2
C     * NTGMAX integer                   maximum number of target points
C     * DEG    integer                   actual degree
C     * NXU    integer                   actual number of Xu points
C     * NTG    integer                   actual number of target points
C     * XU1    double(NXUMAX)            array of the first coordinates of
C     *                                  Xu points
C     * XU2    double(NXUMAX)            array of the second coordinates of
C     *                                  Xu points
C     * WXU    double(NXUMAX)            array of the weights
C     * FVAL   double(NXUMAX)            values of the function at 
C     *                                  Xu points
C     * T1     double(0:DEGMAX,DEGMAX)   Normalized Chebyshev polynomials at
C     *                                  the first coordinate of the points
C     * T2     double(0:DEGMAX,DEGMAX)   Normalized Chebyshev polynomials at 
C     *                                  the second coordinate of the points
C     * C0     double(0:DEGMAX,0:DEGMAX) matrix C0 of the coefficients
C     * TG1    double(NTGMAX)            array of the first coordinates of
C     *                                  target points
C     * TG2    double(NTGMAX)            array of the second coordinates of
C     *                                  target points
C     * INTFTG double(NTGMAX)            values of the interpolated function
C     * F      double function           Franke function at target points
C     * MAP1   double function           first component of the map
C     *                                  [-1,1]^2 -> [0,1]^2
C     * MAP2   double function           second component of the map
C     *                                  [-1,1]^2 -> [0,1]^2
C     * MAXERR double                    maximum norm of the error at
C     *                                  target points
C     * ESTERR double                    estimated error
C     * NTG1   integer                   number of target points for direction
C     ************************************************************************
      write(6,*) '*********************************************'
      write(6,*) 'input function number (1--6): '
      read(5,*) number
      write(6,*) number
      write(6,*) 'input deg: '
      read(5,*) deg
      write(6,*) deg
      IF (DEG .GT. DEGMAX) THEN
         WRITE(6,*) 'IN _DRVCUB: DEG ',DEG,' > DEGMAX ',DEGMAX
         WRITE(6,*) 'STOP'
         STOP
      END IF
      write(6,*) 'input the number of target points for direction: '
      read(5,*) ntg1
      write(6,*) ntg1
      IF (NTG1 .LT. 2) THEN
         WRITE(6,*) 'IN _DRVCUB: NTG',NTG,' < 2'
         WRITE(6,*) 'STOP'
         STOP
      END IF      
      NTG = NTG1 ** 3
      IF (NTG .GT. NTGMAX) THEN
         WRITE(6,*) 'IN _DRVCUB: NTG ',NTG,' > NTGMAX ',NTGMAX
         WRITE(6,*) 'STOP'
         STOP
      END IF      
C     *
C     * Build the Xu points
C     *
      CALL BLDXU(DEG + 1,XU1,XU2,WXU,NXU)
      J = 0
      DO 5 I = 1,NXU
C     *
C     * Compute the function at points mapped to [0,1]^3
C     * See the functions MAP1, MAP2 and MAP3 in order to change the
C     * domain
C     *
         FVAL(i,j) = F(number,MAP1(XU1(I),XU2(I)),MAP2(XU1(I),XU2(I)),
     1        MAP3(cos(j * pi / (deg + 1))))
         w(i,j) = wxu(i) / (deg + 1) / 2
 5    CONTINUE
      do 10 j = 1,deg
         DO 15 I = 1,NXU
C     *
C     * Compute the function at points mapped to [0,1]^3
C     * See the functions MAP1, MAP2 and MAP3 in order to change the
C     * domain
C     *
            FVAL(i,j) = F(number,MAP1(XU1(I),XU2(I)),
     1           MAP2(XU1(I),XU2(I)),MAP3(cos(j * pi / (deg + 1))))
            w(i,j) = wxu(i) / (deg + 1)
 15      CONTINUE
 10   continue
      J = DEG + 1
      DO 20 I = 1,NXU
C     *
C     * Compute the function at points mapped to [0,1]^3
C     * See the functions MAP1, MAP2 and MAP3 in order to change the
C     * domain
C     *
         FVAL(i,j) = F(number,MAP1(XU1(I),XU2(I)),MAP2(XU1(I),XU2(I)),
     1        MAP3(cos(j * pi / (deg + 1))))
         w(i,j) = wxu(i) / (deg + 1) / 2
 20   CONTINUE 

C     *
C     * Compute the matrix C0 of the coefficients
C     *
c      call tim(time,1)
      call hyper3(deg,degmax,nxu,nxumax,xu1,xu2,w,fval,T1,T2,t3,B0,
     1     C0,esterr)
c      call tim(time,2)
c      write(6,*) time
C     *
C     * Build the grid of target points on [-1,1]^3
C     *
      count = 0
      DO 60 I = 1,NTG1
         DO 65 J = 1,NTG1
            do 70 k=1,ntg1
               count = count+1
               TG1(count) = -1.0D0 + (k - 1) * 2.0D0 / (NTG1 - 1)
               TG2(count) = -1.0D0 + (j - 1) * 2.0D0 / (NTG1 - 1)
               TG3(count) = -1.0D0 + (i - 1) * 2.0D0 / (NTG1 - 1)
               intftg(count) = hy3val(deg,degmax,c0,
     1              tg1(count),tg2(count),tg3(COUNT),t1,t2,t3)
 70         continue
 65      CONTINUE
 60   CONTINUE
C     *
C     * Compute the maximum norm error
C     *
      MAXERR = 0.0D0
      DO 90 I = 1,NTG
         MAXERR = MAX(MAXERR,ABS(F(number,MAP1(TG1(I),TG2(I)),
     1        MAP2(TG1(I),TG2(I)),MAP3(TG3(i))) - INTFTG(I)))
 90   CONTINUE   
      WRITE(6,*)
      WRITE(6,1) 'Estimated error:     ',esterr
      WRITE(6,1) 'Maximum error value: ',MAXERR
      WRITE(6,*)
 1    FORMAT(1X,A21,E8.2)
 2    FORMAT(3(1X,E14.8))
      STOP
      END

      DOUBLE PRECISION FUNCTION MAP3(T3)
C     ************************************************
C     * First component of the map [-1,1]^2 -> [0,1]^2
C     ************************************************
      DOUBLE PRECISION T3
      DOUBLE PRECISION A3,B3
      DATA A3/0.0D0/,B3/1.0D0/
C     *
C     * Specify different values for A1,B1 in order to change the
C     * rectangular domain to [A1,B1] x [A2,B2]
C     *
      MAP3 = ((B3 - A3) * T3 + (B3 + A3)) / 2
      RETURN
      END
      DOUBLE PRECISION FUNCTION MAP1(T1,T2)
C     ************************************************
C     * First component of the map [-1,1]^2 -> [0,1]^2
C     ************************************************
      DOUBLE PRECISION T1,T2
      DOUBLE PRECISION A1,B1
      DATA A1/0.0D0/,B1/1.0D0/
C     *
C     * Specify different values for A1,B1 in order to change the
C     * rectangular domain to [A1,B1] x [A2,B2]
C     *
      MAP1 = ((B1 - A1) * T1 + (B1 + A1)) / 2
      RETURN
      END
      DOUBLE PRECISION FUNCTION MAP2(T1,T2)
C     *************************************************
C     * Second component of the map [-1,1]^2 -> [0,1]^2
C     *************************************************
      DOUBLE PRECISION T1,T2
      DOUBLE PRECISION A2,B2
      DATA A2/0.0D0/,B2/1.0D0/
C     *
C     * Specify different values for A2,B2 in order to change the
C     * rectangular domain to [A1,B1] x [A2,B2]
C     *
      MAP2 = ((B2 - A2) * T2 + (B2 + A2)) / 2
      RETURN
      END
